@@ -25,6 +25,8 @@ from astroid.exceptions import AstroidError
 from astroid.nodes._base_nodes import ImportNode, Statement
 from astroid.typing import InferenceResult, SuccessfulInferenceResult
 
+from pylint.constants import TYPING_NEVER, TYPING_NORETURN
+
 if TYPE_CHECKING:
     from functools import _lru_cache_wrapper
 
@@ -1484,7 +1486,6 @@ def node_type(node: nodes.NodeNG) -> SuccessfulInferenceResult | None:
 
 def is_registered_in_singledispatch_function(node: nodes.FunctionDef) -> bool:
     """Check if the given function node is a singledispatch function."""
-
     singledispatch_qnames = (
         "functools.singledispatch",
         "singledispatch.singledispatch",
@@ -1540,7 +1541,6 @@ def find_inferred_fn_from_register(node: nodes.NodeNG) -> nodes.FunctionDef | No
 
 def is_registered_in_singledispatchmethod_function(node: nodes.FunctionDef) -> bool:
     """Check if the given function node is a singledispatchmethod function."""
-
     singledispatchmethod_qnames = (
         "functools.singledispatchmethod",
         "singledispatch.singledispatchmethod",
@@ -1831,7 +1831,11 @@ def is_sys_guard(node: nodes.If) -> bool:
             and value.as_string() == "sys.version_info"
         ):
             return True
-
+    elif isinstance(node.test, nodes.Attribute) and node.test.as_string() in {
+        "six.PY2",
+        "six.PY3",
+    }:
+        return True
     return False
 
 
@@ -2148,7 +2152,9 @@ def is_singleton_const(node: nodes.NodeNG) -> bool:
 
 
 def is_terminating_func(node: nodes.Call) -> bool:
-    """Detect call to exit(), quit(), os._exit(), or sys.exit()."""
+    """Detect call to exit(), quit(), os._exit(), sys.exit(), or
+    functions annotated with `typing.NoReturn` or `typing.Never`.
+    """
     if (
         not isinstance(node.func, nodes.Attribute)
         and not (isinstance(node.func, nodes.Name))
@@ -2163,6 +2169,25 @@ def is_terminating_func(node: nodes.Call) -> bool:
                 and inferred.qname() in TERMINATING_FUNCS_QNAMES
             ):
                 return True
+            # Unwrap to get the actual function node object
+            if isinstance(inferred, astroid.BoundMethod) and isinstance(
+                inferred._proxied, astroid.UnboundMethod
+            ):
+                inferred = inferred._proxied._proxied
+            if (
+                isinstance(inferred, nodes.FunctionDef)
+                and isinstance(inferred.returns, nodes.Name)
+                and (inferred_func := safe_infer(inferred.returns))
+                and hasattr(inferred_func, "qname")
+                and inferred_func.qname()
+                in (
+                    *TYPING_NEVER,
+                    *TYPING_NORETURN,
+                    # In Python 3.7 - 3.8, NoReturn is alias of '_SpecialForm'
+                    "typing._SpecialForm",
+                )
+            ):
+                return True
     except (StopIteration, astroid.InferenceError):
         pass
 
@@ -2272,7 +2297,6 @@ def is_enum_member(node: nodes.AssignName) -> bool:
     """Return `True` if `node` is an Enum member (is an item of the
     `__members__` container).
     """
-
     frame = node.frame()
     if (
         not isinstance(frame, nodes.ClassDef)
